;************************************************************************
;	Volker Schulze 03.08.2017      					*
;									*
; 	Analog to PWM Converter 					*
;                                               	           	*     
;				 	I/O 3 		(Pin 4) 	*
;	Analog in	 		I/O 0 		(Pin 7)		*
;					I/O 5		(Pin 2)		*		                      
;	Motor Drive  			I/O 2	 	(Pin 5) 	*                  
;					I/O 1		(Pin 6) 	*                   
;       1/4-OSC-Freq out		I/O 4		(Pin 3)		*                                                        
;								  	*
;************************************************************************

	list      p=12F683        ; list directive to define processor
	include <P12F683.inc>

	errorlevel  -302          ; suppress message 302 from list file

	__CONFIG   _FCMEN_ON & _IESO_OFF & _CP_OFF & _CPD_OFF & _BOD_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_CLKOUT 
;**************************************************************
;
; 12F683 erzeugt an GP2 ein 244Hz-PWM-Signal
;
; Prozessortakt:  4 MHz intern, Zyklustakt 1 MHz 
;
;**************************************************************
; Variablen festlegen  20h ... 5Fh 

W_save		Equ	0x20		; auch 0xA0
Status_save	equ	0x21		; registers for saving context
Flags		equ	0x22		; flag bits (see definitions below)
PinL		equ	0x23
loops		equ	0x24		; timer fr wait
loops2		equ	0x25		; timer fr wait 
Puls		equ	0x26		;
PinL1		equ	0x27

; Flag bits (FLAGS)
#define	InPulse		GPIO, 3
#define	Out1		GPIO, 5		 
#define	Out2		GPIO, 4
#define	Out3		GPIO, 2		; PWM
#define	Out4		GPIO, 1		; Richtungswechselsperre
#define	Analog		GPIO, 0		; Analogspannung


;***********************************************************************
	org	0x00
	goto	Main			
;***********************************************************************
; die Interruptserviceroutine 
;	ORG     0x04             	; interrupt vector location
	RETFIE
;**************************************************************
; Anfangsinitialisierung
Init

; internen Takt 4MHz
	bsf	STATUS, RP0		; Bank 1
	movlw	0x65			; Clock 4MHz
	movwf	OSCCON		
	bcf	STATUS, RP0		; Bank 0
	CLRF	Flags

; IO-Pins
	bcf	STATUS, RP0		; Bank 0
	clrf	GPIO			; Init GPIO
	movlw	0x07
	movwf	CMCON0			; alle Pins digital (nicht Comp)
	bsf	STATUS, RP0		; Bank 1
	clrf	ANSEL	
	movlw	09h			; GP<3,0> als inputs
	movwf	TRISIO
	bcf	OPTION_REG,7		; GPPU
	movlw	0x32
	movwf	WPU			; pull up ein			
	movlw 	0x71			; ADC Fosc/16 clock and GP0 as analog
	IORWF 	ANSEL 			; 
	bcf	STATUS, RP0		; Bank0

; ADC einschalten
; ADC-Eingang AN0 auswhlen
; Daten linksbndig
	clrf	ADCON0
	MOVLW	0x01
	MOVWF	ADCON0

; PWM vorbereiten
; 4MHz-Takt -> 1 MHz-Zyklustakt
; Vorteiler 1:16 und Timer2 einschalten
	BSF	T2CON,T2CKPS1	; Vorteiler 1:16 TMR2 62.500Hz
	BCF	T2CON,T2CKPS0	; Vorteiler 1:16
	BSF	T2CON,TMR2ON	; Timer2 ein

; Frequenz auf 244Hz einstellen
	BSF	STATUS,RP0	; Bank1
	MOVLW	0xFF		; 255(+1)
	MOVWF	PR2		; 244Hz
	BCF	STATUS,RP0	; Bank0

; Tastverhltnis auf 0%
	MOVLW	0x00
	MOVWF	CCPR1L  	; PWM auf 0%

; PWM MODE mit CCP1 initialisieren, LSB 1:0 auf 0 nur obere 8bit MSB genutzt (Auflsung 256 Schritte)
	CLRF	CCP1CON		; CCP1-Modus aus
	BSF	CCP1CON,CCP1M3	; CCP1-Modus PWM-Mode, active high
	BSF	CCP1CON,CCP1M2	;
	RETURN

;**********************************************************
; Hauptprogrammschleife

Main
	call	Init			; PIC initialisieren
	call 	pause

Start					

;Eingangsspannung wandeln
    BSF      ADCON0, 1     ; ADC starten
loop
    BTFSC    ADCON0, 1     ; ist der ADC fertig?
    GOTO     loop          ; nein, weiter warten

; Wert nach CCPR1L schreiben
    BCF      ADCON0, 1     
    movfw    ADRESH       ; obere 8 Bit auslesen
    movwf    CCPR1L       ; obere 8-Bit nach CCPR1L
    clrf     ADRESH

; Warten, damit der ADC sich erholen kann
    call     wait 		;WAS goto

	goto	Start


;**************************************************************
; Warteschleife 1 ms fr einen 4-MHz-PIC-Takt
wait1ms
        MOVLW   .110           ; Zeitkonstante fr 1ms
        MOVWF   loops2
Wai2    NOP 
        NOP
        NOP
        NOP
        NOP
        NOP
        DECFSZ  loops2, F      ; 1 ms vorbei?
        GOTO    Wai2           ; nein, noch nicht 
	RETURN

;*****************************************************
;Zeitverzgerung um loops * 1 ms
; 4 MHz externer Takt bedeutet 1 MHz interner Takt
; also dauert 1 ms genau 1000 Befehle
; 100 Schleifen a 10 Befehle sind 1000 Befehle = 1 ms

wait
wait_ms		movlw   D'100'         ; timing adjustment variable (1ms)
        	movwf   loops2
wait_ms_loop    nop                    ; warten und nichts tun
        	nop
        	nop
        	nop
		nop
        	nop
        	nop
        	decfsz  loops2, f      ; innere Schleife fertig?
        	goto    wait_ms_loop   ; nein, noch mal
                     			
        	decfsz  loops, f       ; uere Schleife fertig?
        	goto    wait_ms        ; nein, noch mal
        	retlw   0              ; fertig und return

pause
		movlw	D'250'		; 250 ms Pause nach dem Einschalten
		movwf	loops	
		call	wait
		return

;**************************************************************

	end

;**************************************************************
